%% MisMeasurePBS1a: treat IPP investment as intermediate input
% IPP investment produced by PBS sectors Computer Systems Design (NAICS
% 5415), and misc prof, scientific, and technical services (NAICS 5412OP)
% is treated as an intermediate good.
% 
% This script uses 
% (1) nominal investment by industry and type, 'BEA_FA_Industry_Invest_17sec', 
%     previously constructed with script "BEA_FA_Data_Import". 
% (2) IO use tables for the xx sector aggregation, 'IOUse_ComByIndxxSec_1997-2015', 
%     previously constructed using 'io_table_xxSec_1997_2015revised', for xx=17&18
% (3) The 16 private sector aggregation of the 1997 capital flow tables,
%     'CapFlow16Sec_1997'
%
% We treat the purchase of IPP investment produced by PBS
% - by private sector as purchase of intermediate input 
%   -> allocate private IPP investment by industry to intermediate use
% - by government as consumption 
%   -> no adjustment of Intermediate Use table
%
% We construct alternatives for
% (1) nominal IPP investment by industry, 'BEA_FA_Industry_Invest_17sec_Inv2Inter'
% (2) IO use tables, 'IOUse_ComByInd17Sec_1997-2015_Inv2Inter', 
% (3) Investment requirement tables, 'CapFlow16Sec_1997_Inv2Inter' 
% for the 17 sector aggregation (incl gov)
%
% NOTE: IO Use Table in Millions of $s and FA Investment Tables in Billions of $s
% 
% A.Hornstein, 11/4/2021
% revised 12/10/2021: mostly just commentary
% -------------------------------------------------------------------------
inputlocFAT='..\data_bea_fat\DataProcessed\';
inputlocIO ='..\data_io_cap\DataProcessed\';
outputloc='DataProcessed\';
adjname  ='Invest2Inter';

% initialize the alternative investment file for 17 sectors
% (non-residential private+govmnt) 
fname='BEA_FA_Industry_Invest';
load([inputlocFAT fname,'_17sec'],'tt','xxI2','xxE2','xxS2','xname2');
ttX=tt;
save([outputloc, fname,'_17sec_' adjname],'tt','xxI2','xxE2','xxS2','xname2');

% load capital flow matrix for 16 private industries
load([inputlocIO 'CapFlow16Sec_1997'],'omega');

ttA=(2015:2015);
% ttA=(1997:2015);
ntA=length(ttA);
omegaAdj=cell(ntA,1);
for i = 1:ntA
    % use data from Use Tables and FATs for year 'date'
    date=ttA(i);
    
    %% Extract IPP contribution from PBS sectors with large contribution to IPP investment
    % load data from 18 sector decomposition
    % - IPP producing PBS in row 17
    % - final IPP use in column 25
    fname=[inputlocIO 'IOUse_ComByInd18Sec_1997-2015.xlsx'];
    use18T=readtable(fname,'Sheet',num2str(date),'Range','A1', ...
        'ReadVariableNames',true,'ReadRowNames',true);
    use18=use18T.Variables;
    % contribution from IPP producing PBS sectors to private IPP investment 
    xPBSipp=use18(17,25);
    
    %% Allocate private IPP investment that is PBS produced to PBS produced intermediates
    % load data from 17 sector decomposition
    % - PBS in row 12
    % - final use private non-resi investment [S,E,IPP] in column 22:24    
    fname=[inputlocIO 'IOUse_ComByInd17Sec_1997-2015.xlsx'];
    use17T=readtable(fname,'Sheet',num2str(date),'Range','A1', ...
        'ReadVariableNames',true,'ReadRowNames',true);
    use17=use17T.Variables;
    secName=use17T.Properties.VariableNames(1:17);
    % ---------------------------------------------------------------------
    % final use before adjustment
    % total private IPP investment
    useIPP=sum(use17(1:19,24),1);
    % contribution of PBS to total private non-resi investment
    usePBS=sum(use17(12,22:24),2);
    % ---------------------------------------------------------------------
    % final use after adjustment
    % subtract xPBSipp from PBS contribution to IPP investment    
    use17(12,24)=use17(12,24)-xPBSipp;
    % total private IPP investment
    useIPPadj=sum(use17(1:19,24),1);
    use17(29,24)=useIPPadj;
    % contribution of PBS to total private investment
    usePBSadj=sum(use17(12,22:24),2);
    % Adjustment factor for total private IPP investment
    IPPadj=useIPPadj/useIPP;
    % Adjustment factor for contribution of PBS to total private investment
    PBSadj=usePBSadj/usePBS;    
    % ---------------------------------------------------------------------
    %% Allocate xPBSipp to intermediate good use 
    % Distribute to intermediate goods use according to the distribution of
    % overall IPP investment across private sectors 
    % Obtain distribution of IPP investment across private industries
    it =(date==ttX);
    xI =xxI2(it,1:16);
    xIs=xI/sum(xI,2);
    % adjusted contribution of PBS to intermediate good use
    use17(12,1:16)   =use17(12,1:16)   +xIs*xPBSipp;
    use17(12,[18 20])=use17(12,[18 20])+xPBSipp;
    use17T.Variables =use17;
    fname=[outputloc 'IOUse_ComByInd17Sec_1997-2015_' adjname '.xlsx'];
    writetable(use17T,fname,'Sheet',num2str(date),'Range','A1','WriteRowNames',true)
    
    %% adjust the IPP investment by industry
    % scale total private IPP investment based on adjusted total IPP in Use Table
    xxI2(it,1:16)=xI*IPPadj;
    fname=[outputloc 'BEA_FA_Industry_Invest_17sec_' adjname];
    save(fname,'xxI2','-append');
    
    %% adjust the 1997 capital flow tables for 16 private sectors, incl housing
    % rescale PBS contribution to private industry investment (S+E+I)
    omegaA=omega;
    omegaA(12,:)=PBSadj*omegaA(12,:);
    omegaA=omegaA*diag(1./sum(omegaA,1));
    omegaAdj{i}=omegaA;
    
    omegaAT=array2table(omegaA,'VariableNames',secName(1:16),'RowNames',secName(1:16));
    fname=[outputloc 'CapFlow_16Sec_ComByInd_1997-2015_' adjname '.xlsx'];
    writetable(omegaAT,fname,'Sheet',num2str(date),'Range','A1','WriteRowNames',true)

end
tt=ttA;
fname=[outputloc 'CapFlow_16Sec_ComByInd_1997-2015_' adjname];
save(fname,'tt','omegaAdj');