%% MisMeasurePBS2a: treat intermediate inputs as IPP investment
% A fraction of intermediate goods produced by PBS is now treated as an
% investment good. We do not make a distinction between PBS industries that
% do or do not produce IPP.
% 
% This script uses 
% (1) nominal investment by industry and type, 'BEA_FA_Industry_Invest_17sec', 
%     previously constructed with script "BEA_FA_Data_Import". 
% (2) IO use tables for the 17 sector aggregation, 'IOUse_ComByInd17Sec_1997-2015', 
%     previously constructed using 'io_table_17Sec_1997_2015revised'
% (3) The 16 private sector aggregation of the 1997 capital flow tables,
%     'CapFlow16Sec_1997'
%
% We treat a fraction of the intermediate sales by PBS as IPP investment
% produced for the private sector. We do not allocate any of the
% intermediate sales to government investment. This maximizes the impact of
% the reclassification on capital growth.
%
% We construct alternatives for
% (1) nominal IPP investment by industry, 'BEA_FA_Industry_Invest_17sec_Inter2Inv'
% (2) IO use tables, 'IOUse_ComByInd17Sec_1997-2015_Inter2Inv', 
% (3) Investment requirement tables, 'CapFlow16Sec_1997_Inter2Inv' 
% for the 17 sector aggregation (incl gov)
%
% NOTE: IO Use Table in Millions of $s and FA Investment Tables in Billions of $s
% 
% A.Hornstein, 11/4/2021
% revised 12/10/2021: mostly just commentary
% -------------------------------------------------------------------------
inputlocFAT='..\data_bea_fat\DataProcessed\';
inputlocIO ='..\data_io_cap\DataProcessed\';
outputloc='DataProcessed\';
adjname ='Inter2Invest';

% initialize the alternative IPP inv file for 17 sectors
% (non-residential private+govmnt)
fname='BEA_FA_Industry_Invest';
load([inputlocFAT fname,'_17sec'],'tt','xxI2','xxE2','xxS2','xname2');
ttX=tt;
save([outputloc, fname,'_17sec_' adjname],'tt','xxI2','xxE2','xxS2','xname2');

% load capital flow matrix for 16 private industries
load([inputlocIO 'CapFlow16Sec_1997'],'omega');

% fraction of ALL PBS intermediate goods that are treated as investment goods
fx=0.25;

ttA=(2015:2015);
% ttA=(1997:2015);
ntA=length(ttA);
omegaAdj=cell(ntA,1);
for i = 1:ntA
    % use data from Use Tables and FATs for year date
    date=ttA(i);
    % load data from 17 sector decomposition
    % - PBS in row 12
    % - final use private non-resi investment [S,E,IPP] in column 22:24    
    fname=[inputlocIO 'IOUse_ComByInd17Sec_1997-2015.xlsx'];
    use17T=readtable(fname,'Sheet',num2str(date),'Range','A1', ...
        'ReadVariableNames',true,'ReadRowNames',true);
    use17=use17T.Variables;
    secName=use17T.Properties.VariableNames(1:17);
    
    %% allocate PBS produced intermediates to private IPP investment
    % ---------------------------------------------------------------------
    % before adjustment
    % total private IPP investment
    useIPP=sum(use17(1:19,24),1);
    % contribution of PBS to total private non-resi investment
    usePBS=sum(use17(12,22:24),2);
    % total intermediates produced by PBS for private industries
    m0=sum(use17(12,1:16),2);
    % ---------------------------------------------------------------------
    % after adjustment    
    % scale down intermediates
    use17(12,1:16)   =(1-fx)*use17(12,1:16);
    use17(12,[18 20])=use17(12,[18 20])-fx*m0;
    % allocate intermediates to IPP investment
    use17(12,24)=use17(12,24)+fx*m0;
    % total private IPP investment
    useIPPadj=sum(use17(1:19,24),1);
    use17(29,24)=useIPPadj;
    % contribution of PBS to private investment
    usePBSadj=sum(use17(12,22:24),2);   
    % Adjustment factor for total private IPP
    IPPadj=useIPPadj/useIPP;
    % Adjustment factor for contribution of PBS to total private investment
    PBSadj=usePBSadj/usePBS;    
    
    use17T.Variables=use17;
    fname=[outputloc 'IOUse_ComByInd17Sec_1997-2015_' adjname '.xlsx'];
    writetable(use17T,fname,'Sheet',num2str(date),'Range','A1','WriteRowNames',true)
    
    %% adjust the IPP investment by industry
    it=(date==ttX);
    % scale total private IPP investment based on adjusted total IPP in Use Table
    xxI2(it,1:16)=xxI2(it,1:16)*IPPadj;
    fname=[outputloc 'BEA_FA_Industry_Invest_17sec_' adjname];
    save(fname,'xxI2','-append');
    
    %% adjust the 1997 capital flow tables for 16 private sectors, incl housing
    % rescale PBS contribution to private industry investment (S+E+I)
    omegaA=omega;
    omegaA(12,:)=PBSadj*omegaA(12,:);
    omegaA=omegaA*diag(1./sum(omegaA,1));
    omegaAdj{i}=omegaA;    

    omegaAT=array2table(omegaA,'VariableNames',secName(1:16),'RowNames',secName(1:16));
    fname=[outputloc 'CapFlow_16Sec_ComByInd_1997-2015_' adjname '.xlsx'];
    writetable(omegaAT,fname,'Sheet',num2str(date),'Range','A1','WriteRowNames',true)

end
tt=ttA;
fname=[outputloc 'CapFlow_16Sec_ComByInd_1997-2015_' adjname ];
save(fname,'tt','omegaAdj');