% BEAdata_Split_Housing_from_RealEstate
% (1) Import 'Real Estate' (RE) from the IILPA 
% (2) Split the 'Real Estate' (RE) sector into 'Residential Housing' (REH)
%     and 'Other RE' (REO) based on NIPA data for the Housing Sector, Tables 7.4 
%     (a) Split GO, intermediate inputs, and GVA using nominal and real GO,
%         M, and GVA from RE and REH
%     (b) Split labor according to labor compensation shares of REH and REO
%     (c) Split capital according to capital income shares of REH and REO
% (3) Save the split REexH and H sector
%
% Notes
% In the NAICS RE and Rental and Leasing Sector covers 
%  (1) housing (HS), owner-occupied (HSO) and tenant occupied (HST)
%  (2) Other real estate (ORE)
%  (3) Rental and leasing SVCs and lessors of intangible assets (532RL)
% The ILLPAs experimental, 1947-1963 and 1963-2016, and official, 1987-2018, 
% separate out RE (45)=HS+ORE and 532RL (46)
% The supplemental NIPA Housing Sector tables 7.4.x have NIA data for the
% production or HSO, HST, and farm housing. 
% This program tries to separate out ORE from RE using tables 7.4 and
% related data from the FATs and EMP.
% There are two kinks in the labor compensation data from the NIPA Housing
% Sector Tables 7.5: 1960 to 1963 and 1991-92. Since I am using the share
% of labor compensation from NIPA housing in labor compensation from the
% IILPA to allocate IILPA employment to housing and the other RE this leads
% to large changes in housing EMP. I can leave them in and we get some
% pretty large EMP growth rates at the kinks or I can smooth them out.
% Since we are calculating LR trends, It won't hurt if I just smooth
% through the kinks with a narrow window MA. 
%
% A.Hornstein, 1/11/2021 this is a cleaned up version of
%              BEAdata_Split_Housing_from_RealEstate_LongVersion
% -------------------------------------------------------------------------
% redefine real values such that real=nominal in 2012
tNorm=2012;
% -------------------------------------------------------------------------
filelocRaw='DataRaw\';
filelocProc='DataProcessed\';

nfig=0;
%% Step1. load and merge Real Estate: Housing and Other RE Services from KLEMS
% Input data: experimental 44 sector KLEMS, 1947-1963
fname1=[filelocProc 'KLEMSbea_1947_1963_SIC.mat' ];
load(fname1,'SecName','y','v','k','n','m','yq','vq','kq','nq','mq','hrs','tt');
iRE1=32;
SecName{iRE1},
t1=tt;
y1=y(:,iRE1);
v1=v(:,iRE1);
k1=k(:,iRE1);
n1=n(:,iRE1);
m1=m(:,iRE1);
yq1=yq(:,iRE1);
vq1=vq(:,iRE1);
kq1=kq(:,iRE1);
nq1=nq(:,iRE1);
mq1=mq(:,iRE1);
hrs1=hrs(:,iRE1);
% Input data: experimental and official 63 sector KLEMS, 1963-2018, merged
% in 1987
fname2=[filelocProc 'KLEMSbea_1963_1987_2018_NAICS.mat' ];
load(fname2,'SecName','y','v','k','n','m','yq','vq','kq','nq','mq','hrs','tt');
iRE2=45;
SecName{iRE2}
t2=tt;
y2=y(:,iRE2);
v2=v(:,iRE2);
k2=k(:,iRE2);
n2=n(:,iRE2);
m2=m(:,iRE2);
yq2=yq(:,iRE2);

vq2=vq(:,iRE2);
kq2=kq(:,iRE2);
nq2=nq(:,iRE2);
mq2=mq(:,iRE2);
hrs2=hrs(:,iRE2);
% splice data
tS=1963;
[yRE,tt]=splicefun1to2(t1,y1,t2,y2,tS);
vRE=splicefun1to2(t1,v1,t2,v2,tS);
mRE=splicefun1to2(t1,m1,t2,m2,tS);
nRE=splicefun1to2(t1,n1,t2,n2,tS);
kRE=splicefun1to2(t1,k1,t2,k2,tS);
yqRE=splicefun1to2(t1,yq1,t2,yq2,tS);
vqRE=splicefun1to2(t1,vq1,t2,vq2,tS);
mqRE=splicefun1to2(t1,mq1,t2,mq2,tS);
nqRE=splicefun1to2(t1,nq1,t2,nq2,tS);
kqRE=splicefun1to2(t1,kq1,t2,kq2,tS);
hrsRE=splicefun1to2(t1,hrs1,t2,hrs2,tS);

%% Step 2. Load housing sector data from NIPA Supplemental Tables 7.4 
InputFileName =[filelocRaw 'ResidentialHousing11.20.xlsx'];
% common sample 1947-2018
t0=1947;
t1=2018;

% -------------------------------------------------------------------------
% NIPA Table 7.4.5, Millions of current $'s
[data, ~]=xlsread(InputFileName,'GVA-RESI-Nominal','B8:U97');
nt=size(data,1);
tt=1929+(0:nt-1)';
smpl=(t0<=tt)&(tt<=t1);
data=data(smpl,:);
tt=tt(smpl);
% gross output, intermediates, value added, and employee & capital compensation
yH=data(:,1);
mH=data(:,5);
vH=data(:,6);
wH=data(:,12);
kH=vH-wH;

% plot NIPA Housing relative to KLEMS RE
nfig=nfig+1;figure(nfig);clf;
subplot(211);
plot(tt,100*yH./yRE,'r-', ...
    tt,100*vH./vRE,'b-', ...
    tt,100*mH./mRE,'g-', ...
    tt,100*wH./nRE,'m-', ...
    tt,100*kH./kRE,'y-');
title('Share of NIPA HO in BEA KLEMS RE');
legend('GO','GVA','M','N','K');
ylabel('Percent');
subplot(212);
plot(tt,100*mH./yH,'g-', ...
    tt,100*vH./yH,'b-', ...
    tt,100*wH./vH,'m-', ...
    tt,100*kH./vH,'y-');
title('Income Shares for NIPA Housing Sector');
legend('M/GO','GVA/GO','W/GVA','K/GVA');
ylabel('Percent');
% pause

% smooth the kinks in the NIPA labor compensation for the housing sector
wHma=smoothdata(wH,'movmean',5);
nfig=nfig+1;figure(nfig);clf;
plot(tt,log(wH),'r-', ...
    tt,log(nRE),'b-', ...
    tt,log(wHma),'r--');
title('Labor Compensation: NIPA HO vs IILPA RE');
legend('NIPA','IILPA');
wH=wHma;

% -------------------------------------------------------------------------
% NIPA Table 7.4.3, quantity index, 2012=100
[data, ~]=xlsread(InputFileName,'GVA-RESI-Real','B8:L97');
nt=size(data,1);
tt=1929+(0:nt-1)';
smpl=(t0<=tt)&(tt<=t1);
tt=tt(smpl);
data=data(smpl,:);
% gross output, intermediates, and value added
yqH=data(:,1);
mqH=data(:,5);
vqH=data(:,6);
% real=nominal in tNorm
i0=find(tt==tNorm);
yqH=yqH*yH(i0)/yqH(i0);
mqH=mqH*mH(i0)/mqH(i0);
vqH=vqH*vH(i0)/vqH(i0);

% -------------------------------------------------------------------------
%% Step 3. Split Real Estate (45) into 'Res Housing'(REH) and 'RE, other' (REO) 
% -------------------------------------------------------------------------
% Define gross output, VA, and intermediates for RE, excl Housing: 
[yRExH,yqRExH]=DivisiaX(yRE,yH,yqRE,yqH,tt,tNorm);
[vRExH,vqRExH]=DivisiaX(vRE,vH,vqRE,vqH,tt,tNorm);
[mRExH,mqRExH]=DivisiaX(mRE,mH,mqRE,mqH,tt,tNorm);

% -------------------------------------------------------------------------
% Split the KLEMS RE series for labor according to the compensation shares
% from NIPA: this assumes that labor in H and RExH are perfect substitutes 
wHshr=wH./nRE;
nH =wHshr.*nRE;
nqH=wHshr.*nqRE;
nRExH =(1-wHshr).*nRE;
nqRExH=(1-wHshr).*nqRE;
% split hours the same way
hrsH =wHshr.*hrsRE;
hrsRExH=(1-wHshr).*hrsRE;

nfig=nfig+1;figure(nfig);clf;
subplot(211);
plot(tt,log(wH),'r-', tt,log(nRE),'b-');
title('Labor Compensation in Residential Housing and RE: HS<HS+HO');
legend('Housing (HS), NIPA','RE, IILPA (HS+HO)');
ylabel('Millions of Dollars, Log Level');
subplot(212);
plot(tt,wHshr);
title('Labor Compensation in Residential Housing, NIPA');
ylabel('Fraction of IILPA Labor Compensation');
% pause

nfig=nfig+1;figure(nfig);clf;
subplot(211);
plot(tt,log(nH),'r-', tt,log(nRExH),'b-');
title('Labor Compensation in HS and RExHS');
legend('Housing (HS)','REexHS');
ylabel('Millions of Dollars, Log Level');
subplot(212);
plot(tt,log(nqH),'r-', tt,log(nqRExH),'b-');
title('Labor Input in HS and RExHS');
legend('Housing (HS)','RExHS');
ylabel('Millions of 2012 Dollars, Log Level');

% -------------------------------------------------------------------------
% Split the KLEMS RE series for capital services according to the capital
% compensation shares from NIPA: this assumes that capital in H and RExH
% are perfect substitutes 
% Define capital income as GVA-Wage
kH=vH-wH;
kRExH=vRExH-nRExH;
% Split RE capital stock according to the relative capital income
kHshr =kH./(kH+kRExH);
kqH   =    kHshr.*kqRE;
kqRExH=(1-kHshr).*kqRE;

nfig=nfig+1;figure(nfig);clf;
plot(tt,kRE./vRE,'r', tt,kH./vH,'b-', tt,kRExH./vRExH,'g-' )
title('Capital Share in GVA, Nominal');
ylabel('Fraction of GVA');
legend('RE','H','RExH');

nfig=nfig+1;figure(nfig);clf;
subplot(211);
plot(tt,log(kH),'r-', tt,log(kRExH),'b-', tt,log(kRE),'k-')
title('Log Capital Stock, Nominal');
subplot(212);
plot(tt,log(kqH),'r-', tt,log(kqRExH),'b-', tt,log(kqRE),'k-')
title('Log Capital Stock, Real');
legend('H','RExH','RE');

% nfig=nfig+1;figure(nfig);clf;
% subplot(211);
% plot(tt,kH,'r-', tt,kRExH,'b-', tt,kRE,'k-')
% title('Capital Stock, Nominal');
% subplot(212);
% plot(tt,kqH,'r-', tt,kqRExH,'b-', tt,kqRE,'k-')
% title('Capital Stock, Real');
% legend('H','RExH','RE');

% check RE split
% res=[yRE-yRExH-yH, vRE-vRExH-vH, mRE-mRExH-mH, ...
%     kRE-kRExH-kH, nRE-nRExH-nH];
% res=res./[yRE, vRE, mRE, kRE, nRE];

% -------------------------------------------------------------------------
% Step 6. Define the split KLEMS data

y=[yRExH, yH];
v=[vRExH, vH];
m=[mRExH, mH];
k=[kRExH, kH];
n=[nRExH, nH];
yq=[yqRExH, yqH];
vq=[vqRExH, vqH];
mq=[mqRExH, mqH];
kq=[kqRExH, kqH];
nq=[nqRExH, nqH];
hrs=[hrsRExH, hrsH];
SecName={'RE ex Housing','Housing'};

% update TFP growth rates for RE and Housing
[dzy,dzv]=TFPfun(yq,vq,kq,nq,mq,k,n,m);

% check capital income share and intermediate share
% alf=k./v;
% gma=m./y;

% -------------------------------------------------------------------------
% Output data: KLEMS, 1947-2018 for (1) RE ex housing (REO) and (2) Housing
fName='KLEMSbea_RE_Housing_1947_2018';
OutputFileName=[filelocProc fName '.mat'];
save(OutputFileName,'SecName','y','v','k','n','m','yq','vq','kq','nq','mq','dzy','dzv','hrs','tt');

%% ------------------------------------------------------------------------
function [x2,x2q]=DivisiaX(x,x1,xq,x1q,tt,t0)
% DivisiaX: Divisia index for the residual of x after extracting x1
x2=x-x1;
dxq =diff(log(xq));
dx1q=diff(log(x1q));
wx=x./(x-x1);
wx=0.5*(wx(1:end-1)+wx(2:end));
wx1=1-wx;
dx2q=[0;wx.*dxq+wx1.*dx1q];
x2q=exp(cumsum(dx2q));
i0=(tt==t0);
x2q=x2q*x2(i0)/x2q(i0);
end
