%% Table A7: alternative price and TFP growth rates for the PBS sector
% The PBS sectors are 
% IPP producing PBS
%  48 Computer systems design
%  49 Misc profess, scientific, and technical services
% Other PBS
%  47 Legal services
%  50 Mgmnt of companies 
%  51 Admin and support
%  52 Waste mgmnt
% Construct alternative price deflators and output indices and implied
% VA-TFP growth rates for the sectors 48 and 49.
% For the 48 sector use the Byrne, Fernald, and Reinsdorf (2016) liberal
% estimate for custom built software: a weighted average of the BEA price
% index for pre-packaged software (2/3) and the BEA price index for
% custom-built software (1/3)
% For the 49 sector subtract 2 ppts from the KLEMS price index.
%
% A.Hornstein 12/7/2021
% revised 2/22/2022: adapted to new directory structure
% -------------------------------------------------------------------------
tNorm=2012;
inputlocKLEMS='DataProcessed\';
inputlocIPP='..\data_misc\DataRaw\';
outputloc='DataProcessed\';
outputlocTab='..\tables\';


%% import 16 sector aggregates, 1963-2018, for NAICS classification
% PBS is sector 12
fname='KLEMSbea16_1963_1987_2018_NAICS';
load([inputlocKLEMS fname],'tt','yq','y','dzv','SecName');
t16=tt;
y16=y;
yq16=yq;
yp16=100*y16./yq16;
dzv16=dzv;
SecName16=SecName;

%% import 63 sector KLEMS data, 1963-2018, for NAICS classification
% PBS sectors are 
% IPP producing PBS
%  48 Computer systems design
%  49 Misc profess, scientific, and technical services
% Other PBS
%  47 Legal services
%  50 Mgmnt of companies 
%  51 Admin and support
%  52 Waste mgmnt
% We are not using the 1947-1963 sample with 44 sectors since the sector
% 4749 "Professional, scientific, and technical services" combines 
% 47 (Legal) with the IPP sectors 48 & 49
fname='KLEMSbea_1963_1987_2018_NAICS';
load([inputlocKLEMS fname]);
[nt,N]=size(y);
t63=tt;
yp=100*y./yq;

%% import the price deflators for IPP investment types
% Source: BEA NIA 5.6.4
fnameInput=[inputlocIPP 'IPP_NIA_prices.xlsx'];
ippName={ 'IPP',...
    'Software, Total', ...
    'Software, Prepackaged', ...
    'Software, Custom', ...
    'Software, Own Account' ...
    'R&D, Total', ...
    'R&D, Business', ...
    'R&D, Mft', ...
    'R&D, Non-Mft', ...
    'R&D, Scientific', ...
    'R&D, Computer Systems'};
[data,~]=xlsread(fnameInput,'price','A5:L96');
tt=data(:,1);
pp=data(:,2:end);
% common sample with KLEMS data: 1963-2018
smpl=(t63(1)<=tt)&(tt<=t63(end));
tt=tt(smpl);
pp=pp(smpl,:);
[nt,np]=size(pp);

%% alternative price index for KLEMS 48 from NIA software prices
% restricted sample 1985-present: after 1985 BEA provides separate data
% for pre-packaged and custom software
smpl=(tt>=1985);
% redefine price index for Custom Software, follows Byrne, Fernald
% and Reinsdorf (2016): 2/3 pre-packaged and 1/3 custom
dpC=diff(log(pp(smpl,[3 4])))*[2;1]/3;
dpC=cumsum([0;dpC]);
ns =length(dpC);
pC =[NaN(nt-ns,1);exp(dpC)];
% normalize pC such that pC(tNorm)=100
pC =100*pC/pC(tt==tNorm);

figure(1);clf; hold on
plot(tt,log(pp(:,2)),'k-');
plot(tt,log(pp(:,3)),'b-');
plot(tt,log(pp(:,4)),'r-', tt,log(pC),'r--');
plot(tt,log(pp(:,11)),'g-+');
plot(t63,log(yp(:,48)),'g-o');
xlim([1959 2018]);
title('Software');
legend({'Total','Pre-packaged','Custom, BEA','Custom, Alt','Computer Systems, BEA','Computer Systems, KLEMS'});

% average rate of price change for (1-2) NIA Sofware (pre-p&cust), 
% (3) BFR-adj NIA Custom Software, and (4) Comp Sys Design (KLEMS)
dps=[NaN(1,5); diff(log([ pp(:,2:4), pC, yp(:,48) ]))];
smpl=[ 1964 1985; 1986 2000; 2001 2018 ];
ns=size(smpl,1);
dpsAvg=NaN(ns,5);
for j=1:ns
    ii=(smpl(j,1)<=tt)&(tt<=smpl(j,2));
    dpsAvg(j,:)=100*mean(dps(ii,:));
end

varnames={'SW, Total','SW, Pre-pack','SW, Custom','SW, Custom (Alt)','Comp Sys Design, KLEMS'};
rownames={'1964-1985','1986-2000','2001-2018'};
dpsAvgT=array2table(dpsAvg,'VariableNames',varnames,'RowNames',rownames);

%% alternative price index for KLEMS 49 from NIA R&D prices
rdName=cell(7,1);
rdName(1:6)=ippName(6:11);
rdName{7}='Misc Scientific, KLEMS 49';

figure(2);clf; hold on
plot(tt,log(pp(:,6)),'k-');
plot(tt,log(pp(:,7)),'r-');
plot(tt,log(pp(:,8)),'b-');
plot(tt,log(pp(:,9)),'g-');
plot(tt,log(pp(:,10)),'m-+');
plot(tt,log(pp(:,11)),'y-');
plot(t63,log(yp(:,49)),'m-o');
title('R&D');
legend(rdName,'Location','NorthWest');
xlim([1959 2020]);

figure(3);clf; hold on
plot(tt,log(pp(:,6)),'k-');
plot(tt,log(pp(:,7)),'r-');
plot(tt,log(pp(:,8)),'b-');
plot(tt,log(pp(:,9)),'g-');
plot(tt,log(pp(:,10)),'m-+');
plot(tt,log(pp(:,11)),'y-');
plot(t63,log(yp(:,49)),'m-o');
title('R&D');
legend(rdName,'Location','NorthWest');
xlim([1990 2020]);

% average rate of price change for NIA price indices for R&D and KLEMS 49
dprd=[NaN(1,6); diff(log([pp(:,6:10) yp(:,49)])) ];
smpl=[ 1964 1985; 1986 2000; 2001 2018 ];
ns=size(smpl,1);
dprdAvg=NaN(ns,6);
for j=1:ns
    ii=(smpl(j,1)<=tt)&(tt<=smpl(j,2));
    dprdAvg(j,:)=100*mean(dprd(ii,:));
end

varnames=cell(1,6);
varnames(1:5)=ippName(6:10);
varnames{6}='Misc Prof Svcs, KLEMS 49';
rownames={'1964-1985','1986-2000','2001-2018'};
dprdAvgT=array2table(dprdAvg,'VariableNames',varnames,'RowNames',rownames);

%% alternative aggregates for PBS
ypA=yp;
% KLEMS 48 price adjustment
% use alternative price index for custom-built software for post-1985, 
% pC(tNorm)=100 
ypA(t63>=1985,48)=pC(tt>=1985);
% use software price index for pre-1985, splice in 1985
ypA(t63<1985,48)=pp(tt<1985,2)*pC(tt==1985)/pp(tt==1985,2);
% 1964-1985: The alt price index (total software) grows 6 ppts less than than KLEMS 48
% KLEMS 49 price adjustment
% subtract 2 ppt from price inflation of Misc Scientific Services
% NIA R&D deflators grow 0.5 ppts less than KLEMS 49 price index, but
% adjustment for KLEMS 48 is of magnitude 6 ppts
rdum=exp(cumsum([0; diff(log(ypA(:,49)))-0.02]));
ypA(:,49)=100*rdum/rdum(t63==tNorm);
% modified real gross output
yqA=y./(ypA/100);
% modified real value added
[~,vqA]=VAfun(y,m,yqA,mq,t63,tNorm);

% plot prices and quantities for KLEMS 48 and 49, actual and modified
figure(10);clf;hold on
i=48;
subplot(221)
plot(t63,log(yp(:,i)),'k-', t63,log(ypA(:,i)),'r-');
title([ SecName{i} ' : Price Index']);
subplot(222)
plot(t63,log(yq(:,i)),'k-', t63,log(yqA(:,i)),'r-');
title([ SecName{i} ' : Quantity Index']);
i=49;
subplot(223)
plot(t63,log(yp(:,i)),'k-', t63,log(ypA(:,i)),'r-');
title([ SecName{i} ' : Price Index']);
subplot(224)
plot(t63,log(yq(:,i)),'k-', t63,log(yqA(:,i)),'r-');
title([ SecName{i} ' : Quantity Index']);
legend({'KLEMS','Modified'},'Location','SouthEast');

% intermediate step 1: gross output shares of KLEMS 48&49 in overall PBS
y4849inPBS =100*sum(y(:,[48 49]),2)./sum(y(:,(47:52)),2);
y48in4849=100*y(:,48)./sum(y(:,[48 49]),2);
figure(11);clf;hold on;
plot(t63,y4849inPBS,'r-');
plot(t63,y48in4849,'b--');
title('PBS Shares, Gross Output Shares');
legend('48 and 49 in PBS','48 in 48 and 49');

% intermediate step 2: aggregate KLEMS 48&49 and compare actual and modified
idx=[48 49];
dyp =100*diff(log(yp(:,idx)));
dypA=100*diff(log(ypA(:,idx)));
ysC =y(:,idx)./kron(sum(y(:,idx),2),ones(1,2));
ysC =(ysC(2:end,:)+ysC(1:end-1,:))/2;
dypS=ysC.*dyp;
dypSA=ysC.*dypA;
figure(11);clf;
subplot(221);hold on;
plot(t63(2:end),sum(dypS,2),'k-');
plot(t63(2:end),dyp(:,1),'k--');
plot(t63(2:end),dyp(:,2),'k:');
plot(t63(2:end),sum(dypSA,2),'r-');
plot(t63(2:end),dypA(:,1),'r--');
plot(t63(2:end),dypA(:,2),'r:');
title('Inflation Rates');
subplot(222);hold on;
plot(t63(2:end),sum(dypS,2),'k-');
plot(t63(2:end),dypS(:,1),'k--');
plot(t63(2:end),dypS(:,2),'k:');
plot(t63(2:end),sum(dypSA,2),'r-');
plot(t63(2:end),dypSA(:,1),'r--');
plot(t63(2:end),dypSA(:,2),'r:');
title('Inflation Rates: Contributions');
legend({'48 and 49','48: Comp Sys Design','49: Misc Prof Svcs'});
subplot(223);hold on;
plot(t63(2:end),sum(dypS,2)-sum(dypSA,2),'k-');
plot(t63(2:end),dypS(:,1)-dypSA(:,1),'k--');
plot(t63(2:end),dypS(:,2)-dypSA(:,2),'k:');
title('Inflation Rates, Actual - Modified: Contributions');
subplot(224);hold on;
rdum=sum(dypS,2)-sum(dypSA,2);
plot(t63(2:end),100*(dypS(:,1)-dypSA(:,1))./rdum,'k--');
plot(t63(2:end),100*(dypS(:,2)-dypSA(:,2))./rdum,'k:');
title('Inflation Rates, Actual - Modified: Contribution Share');
pause;

% alternative PBS aggregate from KLEMS 47-52
idx=(47:52);
% Gross output aggregate, nominal and real
[y1,y1qA]=DivisiaQ(y(:,idx),yqA(:,idx),t63,tNorm);
y1pA=100*y1./y1qA;
% Capital aggregate, nominal and real
[k1,k1q]=DivisiaQ(k(:,idx),kq(:,idx),t63,tNorm);
% Labor aggregate, nominal and real
[n1,n1q]=DivisiaQ(n(:,idx),nq(:,idx),t63,tNorm);
% Intermediate input aggregate, nominal and real
[m1,m1q]=DivisiaQ(m(:,idx),mq(:,idx),t63,tNorm);

% Value added, nominal and real, for the PBS components KLEMS 47-52
[v1,v1qA]=VAfun(y(:,idx),m(:,idx),yqA(:,idx),mq(:,idx),t63,tNorm);
% Value added, nominal and real, for the PBS aggregate
[v1,v1qA]=DivisiaQ(v1,v1qA,t63,tNorm);
% Value added TFP for the PBS aggregate
[~,dzv1A]=TFPfun(y1qA,v1qA,k1q,n1q,m1q,k1,n1,m1);

% plot KLEMS and modified prices and quantities for the PBS aggregate
figure(11);clf;hold on
i=12;
subplot(321)
plot(t63,log(yq16(:,i)),'k-', t63,log(y1qA),'r-');
title('PBS Quantity Index : Log Level');
legend({'KLEMS','Alternative'},'Location','SouthEast');
subplot(322)
plot(t63(2:end),100*diff(log(yq16(:,i))),'k-', t63(2:end),100*diff(log(y1qA)),'r-');
title('PBS Quantity Index : Growth Rate');
subplot(323)
plot(t63,log(yp16(:,i)),'k-', t63,log(y1pA),'r-');
title('PBS Price Index : Log Level');
subplot(324)
plot(t63(2:end),100*diff(log(yp16(:,i))),'k-', t63(2:end),100*diff(log(y1pA)),'r-');
title('PBS Price Index : Growth Rate');
subplot(325)
plot(t63,100*dzv16(:,i),'k-', t63,100*dzv1A,'r-', t63,100*(dzv1A-dzv16(:,i)),'r-');
legend({'KLEMS','Alternative','KLEMS-Alt'},'Location','SouthEast');
title('PBS VA TFP : Growth Rate');
subplot(326)
plot(t63(2:end),100*diff(log(yp16(:,i)))-100*diff(log(y1pA)),'r-');
title('PBS Price Index :  Growth Rate, KLEMS-Alternative');


%% merge the pre-1963 data with the adjusted post-1963 data
% import 16 sector aggregates, 1947-1963, for SIC classification
fname='KLEMSbea16_1947_1963_SIC';
load([inputlocKLEMS fname],'tt','yq','y','dzv');
tSIC=tt;
% select PBS: 12-th sector
y2p =y(:,12)./yq(:,12);
dzv2=dzv(:,12);
% 1964-1985: alternative price index grows 1 ppt less
% subtract 1 ppt from price inflation in pre-63 sample
y2pA=exp(cumsum([0;diff(log(y2p))-0.01]));
y2pA=y1pA(1)*y2pA/y2pA(end);
ypPBS=[y2pA(1:end-1);y1pA];
% 1964-1985: alternative VA-TFP grows 1.5 ppts faster
% add 1.5 ppts to VA-TFP growth in pre-63 sample
dzv2A=dzv2+0.015;
dzvPBS=[dzv2A(1:end-1);dzv1A];

% merged sample for PBS
tPBS=[tSIC(1:end-1);t63];

% load the original merged data, 1947-2018
fname='KLEMSbea16_1947_1963_1987_2018';
load([inputlocKLEMS fname],'tt','yq','y','dzv','SecName');
% select PBS: 12-th sector
yp =100*y(:,12)./yq(:,12);
dzv=dzv(:,12);

% PBS systematic price measurement error
epsPBS=diff(log(ypPBS))-diff(log(yp));

fname='KLEMS_1947_2018_PBS_modified';
save([outputloc fname],'tPBS','ypPBS','dzvPBS','epsPBS');

figure(12);clf;
subplot(311)
plot(tt(2:end),100*diff(log(yp)),'k-', tPBS(2:end),100*diff(log(ypPBS)),'r-');
title('Price Index, Rate of Change, Percent');
legend({'BEA','Alternative'});
xlim([tt(2) tt(end)]);
grid on;
subplot(312)
plot(tPBS(2:end),100*epsPBS,'r-');
title('Price Index: Alternative - BEA, Percentage Points');
xlim([tt(2) tt(end)]);
grid on;
subplot(313);
plot(tPBS,100*dzvPBS,'r-', tt,100*dzv,'k-');
title('VA TFP Growth Rate, Percent');
xlim([tt(2) tt(end)]);
grid on;

% Table: average rate of price change and TFP growth for PBS, percent
zz=100*[[0 0; diff(log([yp,ypPBS]))], dzv, dzvPBS];
smpl=[ 1948 1963; 1964 1985; 1986 2000; 2001 2018 ];
ns=size(smpl,1);
zzAvg=NaN(ns,4);
for j=1:ns
    ii=(smpl(j,1)<=tt)&(tt<=smpl(j,2));
    zzAvg(j,:)=mean(zz(ii,:));
end

varnames={'KLEMS Price','Alt Price','KLEMS TFP','Alt TFP'};
rownames={'1948-1963','1964-1985','1986-2000','2001-2018'};
zzAvgT=array2table(zzAvg,'VariableNames',varnames,'RowNames',rownames);

fname=[outputlocTab 'TableA7.xlsx'];
writetable(zzAvgT,fname,'Range','A1','WriteRowNames',true)


%% print table in TEX format
fmt=cell(8,1);
fmt{1}='\\begin{tabular}{l|cc|cc} \n';
fmt{2}='\\hline\\hline  \n';
fmt{3}='&\\multicolumn{2}{c|}{Prices}&\\multicolumn{2}{c}{VA TFP} \\\\ \n';
fmt{4}=' &KLEMS & Modified & KLEMS & Modified \\\\ \n';
fmt{5}='\\hline \n';
fmt{6}='%s &  %4.1f & %4.1f & %4.1f & %4.1f \\\\ \n';
fmt{7}='\\hline\\hline \n';
fmt{8}='\\end{tabular} \n';

fname=[outputlocTab 'TableA7.tex'];
fid=fopen(fname,'w');
for i=1:5
    fprintf(fid,fmt{i});
end
for i=1:4
    fprintf(fid,fmt{6},rownames{i},zzAvg(i,:));
end
for i=7:8
    fprintf(fid,fmt{i});
end
fclose(fid);




