% capflow_table_16Sec_1997.m
% 
% Takes the BEA capital flow table from 1997 and combines industry
% definitions to match the 16 private KLEMS industries. 
% This script separates out residential housing from office structures in
% Real Estate. 
% 
% 2017-Oct-06
% Andrew T. Foerster
% revised A.Hornstein, 12/11/2019
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %
% -------------------------------------------------------------------------
% -- Setup -- %
clear;
close all;
clc;

N = 16;
date=1997;

inputloc='DataRaw\';
outputloc='DataProcessed\';

% -- Import Capital Flow Table -- %
flow = xlsread([inputloc 'flow1997.xls'],'180x123Combined','D5:DV183');

% -- Separating Housing and Other Real Estate -- %
re = flow(:,91);
% -------------------------------------------------------------------------
% RE includes office and residential structures, but it does not include
% hotels
% -------------------------------------------------------------------------
% investment going to non-residential office structures
% 26      comm structures
% 40      office building
% 74:77   office machinery
% 79      ind blowers
% 100:107 computers, electronics, communication
% 152:156 office and institutional furniture
% 165     signs
o_ind   = [26 40:42 74:77 79 100:107 152:156 165]; 
% Compare with Andrew's allocation: 
% o_ind   = [23 26 32 40:42 60 74:77 152:156 163];   
% -------------------------------------------------------------------------
% investment going to residential structures
% 5:9     residential structures
% 27      farm housing
% 37:38   maintenance and repair
% 44      mobile homes
% 117:123 household tools
% 147:151 hh furniture
% 157:158 mattresses and blinds
h_ind   = [5:9 27 37:38 44 61 117:123 147:151 157:158]; 
% Compare with Andrew's allocation
% h_ind   = [5:9 27 37:38 44 61 117:123 147:151 157:158 176]; 
% -------------------------------------------------------------------------
% investment not allocated
% 23    water supply, 2/3's water supply inv goes to utilities (13)
% 32    highways, bridges, horizontal structures
% 41:42 fabrics and rugs
% 46:49 foam, rubber, tools, saws
% 53&54 tanks and boilers
% 56:61 parts, e.g. hardware, and lawn and garden equipment
% 80    HVAC
% 94    hand power tools
% 132:133 automobiles and trucks
% 137 travel trailers and campers
% 139&146 aircraft and other transportation equipment
% 163     sporting goods
% 167     miscellaneous mft goods
% 168:173 wholesale & retail trade, transportation
% 174   software publishers
% 176   offices of RE brokers
% 177:179 professional svc's
oh_ind=[ 23 32 41:42 46:49 53:54 56:61 80 94 132:133 137 139 146 163 167:174 176:179 ];

% indices for investment that is not allocated to either office or residential
x_ind   = setdiff(1:size(flow,1),union(o_ind,h_ind));

ore = zeros(size(flow,1),1);
hou = zeros(size(flow,1),1);
ore(o_ind) = re(o_ind);
hou(h_ind) = re(h_ind);
% AF: split unallocated investment equally between office RE and residential RE
ore(x_ind) = 0.5*re(x_ind);
hou(x_ind) = 0.5*re(x_ind);

% AH: allocate x_ind such that we get the 97 ratio of residential to office
% structures from FATs
% 1997 total investment in RE in this capital flow table $316.725 Bill
% 1997 private residential investment from FATS $375.6 Bill
% 1997 investment in office structures from FATS $37.7 Bill
% oreS=sum(ore);
% houS=sum(hou);
% xS  =sum(re(x_ind));
% oShr=37.7/(37.7+375.6);
% o2x =( oShr*(oreS+houS+xS) - oreS )/xS;
% ore(x_ind) = o2x*re(x_ind);
% hou(x_ind) = (1-o2x)*re(x_ind);

%%
% flow(i,j): purchase of i-th new capital good (investment) in j-th industry

% -- Combine Columns -- %
flow = [sum(flow(:, 1: 5),2) ...        % Agriculture
        sum(flow(:, 6:10),2) ...        % Mining
        sum(flow(:,11:13),2) ...        % Utilities
            flow(:,14:14)    ...        % Construction
        sum(flow(:,[22 35:67]),2) ...   % Durable Goods (note order change)      
        sum(flow(:,[15:21 23:34]),2) ...% Nondurable Goods
            flow(:,68)...               % Wholesale Trade
            flow(:,69)...               % Retail Trade
        sum(flow(:,70:78),2) ...        % Transportation and Warehousing
        sum(flow(:,79:86),2) ...        % Information
        sum([flow(:,[87:90 92:95]) ore],2) ... % FIRE excl RESI
        sum(flow(:,96:109),2) ...       % Professional Business Services
        sum(flow(:,110:114),2) ...      % Education and Healthcare
        sum(flow(:,115:118),2) ...      % Arts, Entertainment, Food Serv
        sum(flow(:,119:123),2)...       % Other Services
        hou ...                         % Housing
        ];

% -- Combine Rows -- %
flow = [zeros(1,N);...                      % Agriculture
        sum(flow(1:4,:),1);...              % Mining
        zeros(1,N);...                      % Utilities
        sum(flow(5:40,:),1);...             % Construction
        sum(flow([43:44 48:167],:),1);...   % Durable Goods
        sum(flow([41:42 45:47],:),1);...    % Nondurable Goods
        flow(168,:);...                     % Wholesale Trade
        flow(169,:);...                     % Retail Trade
        sum(flow(170:173,:),1);...          % Transportation and Warehousing
        sum(flow(174:175,:),1);...          % Information      
        flow(176,:);...                     % Finance and Insurance
        sum(flow(177:179,:),1);...          % Professional Business Services
        zeros(1,N);...                      % Education and Healthcare
        zeros(1,N);...                      % Arts, Entertainment, Food Serv
        zeros(1,N);...                      % Other Services
        zeros(1,N)];                        % Housing

    % industry names for aggregated sectors, shortened
secName=cell(N,1);i=1;
secName{i}='Agri';i=i+1;
secName{i}='Mining';i=i+1;
secName{i}='Utilities';i=i+1;
secName{i}='Construction';i=i+1;
secName{i}='DURGoods';i=i+1;
secName{i}='NDRGoods';i=i+1;
secName{i}='Wholesale';i=i+1;
secName{i}='Retail';i=i+1;
secName{i}='Transportation';i=i+1;
secName{i}='Information';i=i+1;
secName{i}='FinancInsurance';i=i+1;
secName{i}='ProfSvcs';i=i+1;
secName{i}='EduHealth';i=i+1;
secName{i}='Arts';i=i+1;
secName{i}='OtherSvcs';i=i+1;
secName{i}='Housing';

% -- Transform to Shares -- %
% flow(i,j): share of i-th new capital good purchase in total j-th industry
% investment 
flow = flow./sum(flow);

flowT=array2table(flow,'VariableNames',secName,'RowNames',secName);
fname=[outputloc 'Capital_Flow_Matrix_16sec_ComByInd.xlsx'];
writetable(flowT,fname,'Sheet',num2str(date),'Range','A1','WriteRowNames',true)

% -- Export to Excel -- %
xlswrite([outputloc 'Capital Flow Matrix 16Sec.xlsx'],flow,'1997','C3')

% -- Calibration Parameters -- %
omega = flow;
save([outputloc 'CapFlow16Sec_1997'],'omega');

