% io_table_18Sec_1997_2015revised.m
% 
% Takes the 1996-2015 BEA Input-Output Make and Use tables with 71 Ind's
% and 71 Commodities and combines industry and commodity definitions to
% match the 18-sector aggregation of KLEMS industries.
% 
% 18 sectors = 16 private industries + housing + govmnt
% FIRE is split into FIN ex RE and Housing.  
% PBS is split into IPP producing sectors, Computer Systems Design (NAICS
% 5415), and misc prof, scientific, and technical services (NAICS 5412OP)
% and the rest of PBS 
% 
% We use the After-Redefinitions IO matrices
% Data in millions of $s
%
% We construct 
% (1) commodity-by-industry use tables, 'IOUse_ComByInd18Sec_1997-2015.xlsx'
% (2) commodity-by-industry make tables, 'IOMake_ComByInd18Sec_1997-2015.xlsx'
% (3) industry-by-industry use tables, 'IOUse_IndByInd18Sec_1997-2015.xlsx'
% (4) industry intermediate input requirement tables, 'IOphi_IndByInd18Sec_1997-2015_final.xlsx'
% (5) industry investment requirement tables, 'IOomg_IndByInd18Sec_1997-2015_final.xlsx'
% (6) industry parameters for growth model, 'IOpar_IndByInd18Sec_1997-2015_final.xlsx' 
%      and 'IO_IndByInd18Sec_xxxx' (incl govmnt)
%      and 'IO_IndByInd18Sec_xxxx_final' (excl govmnt)
% 
% A. Hornstein, 12/12/2019. Based on io_table_19 by Andrew T. Foerster
% revised 11/1/2021: add final use requirements for major investment
% components
% revised 12/10/2021: mostly just commentary
% -------------------------------------------------------------------------
% -- Setup -- %
clear;
close all;
clc;
% --- Data path --- %
% PC path
inputlocIO ='DataRaw\';
inputlocFAT='..\data_bea_fat\DataProcessed\';
outputloc  ='DataProcessed\';

% -- Industry Codes and Labels -- %
usefile =[inputlocIO 'IOUse_After_Redefinitions_PRO_1997-2015_Summary.xlsx'];
makefile=[inputlocIO 'IOMake_After_Redefinitions_1997-2015_Summary.xlsx'];
[~, ~, indname] = xlsread(usefile,'1997','C6:BX6');
[~, ~, indcode] = xlsread(usefile,'1997','C7:BX7');

[~, ~, rawuseCname]  = xlsread(usefile,'1997','C7:CV7');
[~, ~, rawuseRname]  = xlsread(usefile,'1997','B8:B90');
[~, ~, rawmakeCname] = xlsread(makefile,'1997','C7:BX7');
[~, ~, rawmakeRname] = xlsread(makefile,'1997','B8:B79');

% Aggregation of industries/commodities to sectors
N = 18;
agg=cell(N,1);
% 1. Agriculture 
agg{1}=(1:2);
% 2. Mining
agg{2}=(3:5);
% 3. Utilities
agg{3}=6;
% 4. Construction
agg{4}=7;
% 5. Durable Goods
agg{5}=(8:18);
% 6. Nondurable Goods
agg{6}=(19:26);
% 7. Wholesale Trade
agg{7}=27;
% 8. Retail Trade
agg{8}=(28:31);
% 9. Transportation and Warehousing
agg{9}=(32:39);
% 10. Information
agg{10}=(40:43);
% 11. Finance and Insurance
agg{11}=[(44:47) 49 50];
% 12. Prof and Business Services, ex IPP producing sectors
agg{12}=[51 54 55 56];
% 13. Education and Health
agg{13}=(57:61);
% 14. Arts, Entertainment, and Food Serv
agg{14}=(62:65);
% 15. Other Services
agg{15}=66;
% 16. Real estate (Housing)
agg{16}=48;
% 17. IPP producing PBS
agg{17}=[52 53];
% 18. Government
agg{18}=(67:71);

aggUR=agg;
aggUC=agg;
aggMR=agg;
aggMC=agg;

% industry names for aggregated sectors
secName=cell(N,1);i=1;
secName{i}='Agriculture et al';i=i+1;
secName{i}='Mining';i=i+1;
secName{i}='Utilities';i=i+1;
secName{i}='Construction';i=i+1;
secName{i}='Durable Goods';i=i+1;
secName{i}='Nondurable Goods';i=i+1;
secName{i}='Wholesale Trade';i=i+1;
secName{i}='Retail Trade';i=i+1;
secName{i}='Transp and Warehousing';i=i+1;
secName{i}='Information';i=i+1;
secName{i}='Fin and Ins';i=i+1;
secName{i}='PBS, ex IPP prod';i=i+1;
secName{i}='Edu and Health Care';i=i+1;
secName{i}='Arts and Enter';i=i+1;
secName{i}='Other Services, excl Gov';i=i+1;
secName{i}='Housing Services';i=i+1;
secName{i}='PBS, IPP prod';i=i+1;
secName{i}='Government';
% industry names for aggregated sectors, shortened
secNameS=cell(N,1);i=1;
secNameS{i}='Agri';i=i+1;
secNameS{i}='Mining';i=i+1;
secNameS{i}='Utilities';i=i+1;
secNameS{i}='Construction';i=i+1;
secNameS{i}='DURGoods';i=i+1;
secNameS{i}='NDRGoods';i=i+1;
secNameS{i}='Wholesale';i=i+1;
secNameS{i}='Retail';i=i+1;
secNameS{i}='Transportation';i=i+1;
secNameS{i}='Information';i=i+1;
secNameS{i}='FinancInsurance';i=i+1;
secNameS{i}='PBS, ex IT Svcs';i=i+1;
secNameS{i}='EduHealth';i=i+1;
secNameS{i}='Arts';i=i+1;
secNameS{i}='OtherSvcs';i=i+1;
secNameS{i}='Housing';i=i+1;
secNameS{i}='PBS, IPP prod';i=i+1;
secNameS{i}='Government';
% shorten the column labels of the use matrix, non-intermediate input use
% part: there is an upper bound for column names in table arrays
rawuseCnameS=rawuseCname;
rawuseCnameS(71+1:end)= {'Intermed Selected'; ...
    'Intermed Not Selected'; ...
    'Total Intermed'; ...
    'PCE'; ...
    'Nonres, structures'; ...
    'Nonres, eqmnt'; ...
    'Nonres, IPP'; ...
    'Resi'; ...
    'Inventories'; ...
    'Exports'; ...
    'Imports'; ...
    'Fed def: cons'; ...
    'Fed def: struc';...
    'Fed def: eqmnt'; ...
    'Fed def: IPP'; ...
    'Fed nond: cons'; ...
    'Fed nond: structures'; ...
    'Fed nond: eqmnt'; ...
    'Fed nond: IPP'; ...
    'S and L: cons'; ...
    'S and L: struct'; ...
    'S and L: eqmnt'; ...
    'S and L: IPP'; ...
    'GDP: Selected'; ...
    'GDP: Not Selected'; ...
    'GDP: Total'; ...
    'Total Commodity Output'};

% --- import industry investment by type: E, S, & IPP  ----
% investment excl residential structures
fname=[inputlocFAT 'BEA_FA_Industry_Invest_18Sec' ];
load(fname,'tt','xxI2','xxE2','xxS2','xname2');
ttFA=tt;
% residential structures
fname=[inputlocFAT 'BEA_FA_RESI_Invest'];
load(fname,'tt','xxR','xnameR');
xxR=xxR(:,1);
xnameR=xnameR{1};
ttR=tt;

% -- Dates and Sector Number -- %
% tt=(1997:2015);
tt=(2015:2015);
nt=length(tt);
for i = 1:nt
    date=tt(i);
    
    %% Import 71 sector make and use tables
    % -- Import Use Table -- %
    [~, ~, rawuse]  = xlsread(usefile,num2str(date),'C8:CV90');
    rawuse(cellfun(@(x) ~isempty(x) && isnumeric(x) && isnan(x),rawuse)) = {''};
    rawuse(cellfun(@(x) ~isnumeric(x) && ~islogical(x),rawuse)) = {0}; % Replace non-numeric cells
    rawuse=cell2mat(rawuse);

    % -- Import Make Table -- %
    [~, ~, rawmake] = xlsread(makefile,num2str(date),'C8:BX79');   
    rawmake(cellfun(@(x) ~isempty(x) && isnumeric(x) && isnan(x),rawmake)) = {''};
    rawmake(cellfun(@(x) ~isnumeric(x) && ~islogical(x),rawmake)) = {0}; % Replace non-numeric cells
    rawmake=cell2mat(rawmake);

    %% Aggregate from Raw Definitions to New Industry Definitions -- %
    [use,useRname,useCname]   =aggXX(rawuse,rawuseRname,rawuseCnameS,aggUR,aggUC,secName);
    [make,makeRname,makeCname]=aggXX(rawmake,rawmakeRname,rawmakeCname,aggMR,aggMC,secName);

    % -- Export to Excel -- %
    useT=array2table(use,'VariableNames',useCname,'RowNames',useRname);
    fname=[outputloc 'IOUse_ComByInd18Sec_1997-2015.xlsx'];
    writetable(useT,fname,'Sheet',num2str(date),'Range','A1','WriteRowNames',true)
    makeT=array2table(make,'VariableNames',makeCname,'RowNames',makeRname);
    fname=[outputloc 'IOMake_ComByInd18Sec_1997-2015.xlsx'];
    writetable(makeT,fname,'Sheet',num2str(date),'Range','A1','WriteRowNames',true)

    %% Transformation from Commodity-By-Industry to Industry-By-Industry --
    % components of make-use matrices as defined in BEA (2012) CH.12
    % use components
    U =use(1:N+2,1:N);         % intermediate goods part, incl scraps and noncomps
    e =use(1:N+2,N+4:N+23);    % final use, inclusive scrap and noncomparables
    qU=use(1:N+2,end);         % commodity gross output from use
    gU=use(end,1:N);           % industry gross output from use
    va=use(N+2+[(4:6) 9],1:N); % value added and components
    % make components
    g=make(1:N,N+3);        % industry gross output from Make, g=qI
    V=make(1:N,1:N+2);      % industry comm production, incl srcaps and noncomps
    qM=make(N+1,1:N+2);     % commodity gross output from make
    % -- Direct Requirement Table: Share of Inputs, Columns Sum to 1 -- %
    B=U*diag(1./gU);    
    % -- Industry Market Share Table: Industry Share of Total Commodity Production, Columns Sum to 1 -- %
    D=V*diag(1./qM);    
    % industry-by-industry use and requirement matrix
    Uii=D*U;
    Bii=D*B;
    % industry-by-final demand requirement matrix
    ei =D*e;
    eis=ei*diag(1./sum(ei,1));    
    
    % industry-by-industry use table
    useII=zeros(N+4+1,N+20+1);
    useII(1:N,1:N)=Uii;     % intermediate inputs
    useII(1:N,N+(1:20))=ei; % final use
    useII(1:N,N+21)=gU;     % industry gross output
    useII(N+(1:4),1:N)=va;  % value added
    useII(N+5,1:N)=gU;      % industry gross output
    % check useII: rows (columns) sum to final row (column)
    rdumR=100*(sum(useII([(1:N) N+4],1:N),1)./useII(N+5,1:N) -1);
    rdumC=100*(sum(useII(1:N,1:N+20),2)./useII(1:N,N+21) -1);
    
    useIIRname=useRname(1:N+5);
    useIIRname(N+(1:4))=useRname(N+2+[(4:6) 9]);
    useIIRname(N+5)=useRname(length(useRname));
    useIICname=useCname(1:N+20+1);
    useIICname(N+(1:20))=useCname(N+3+(1:20));
    useIICname(N+21)=useRname(length(useRname));
    useIIT=array2table(useII,'VariableNames',useIICname,'RowNames',useIIRname);
    fname=[outputloc 'IOUse_IndByInd18Sec_1997-2015.xlsx'];
    writetable(useIIT,fname,'Sheet',num2str(date),'Range','A1','WriteRowNames',true)
    
    %% Calibration Parameters -- %
    gma  =(va(end,:)./gU)';    
    phi  =Bii*diag(1./sum(Bii,1));
    theta=eis(:,1);
    vaShr=(va(end,:)./sum(va(end,:),2))';
    alpha=( va(3,:)./sum(va([1 3],:),1) )';
    
    % industry contributions to investment types  
    omgS =eis(:,2);
    omgE =eis(:,3);
    omgI =eis(:,4);
    omgR =eis(:,5);
    % industry contributions to industry investment
    j=(ttFA==date);  
    xxSEI =[xxS2(j,:); xxE2(j,:); xxI2(j,:) ]; 
    omgSEI=[omgS,omgE,omgI];  
    omg=zeros(N,N);
    for j=1:3
        omg=omg+kron(omgSEI(:,j),xxSEI(j,:));
    end
    % now add resi inv to investment in housing sector (N-2)
    j=(ttR==date); 
    omg(:,N-2)=omg(:,N-2)+xxR(j)*omgR;
    omg=omg*diag(1./sum(omg,1));

    filename = [outputloc 'IO_IndByInd18Sec_' num2str(date)];
    save(filename,'phi','gma','theta','vaShr','alpha','omgE','omgS','omgI','omgR','omg');       

    % table representation of share parameters
    % phi
    phiT=array2table(phi,'VariableNames',secName,'RowNames',secName);
    fname=[outputloc 'IOphi_IndByInd18Sec_1997-2015.xlsx'];
    writetable(phiT,fname,'Sheet',num2str(date),'Range','A1','WriteRowNames',true)
    % omg
    omgT=array2table(omg,'VariableNames',secName,'RowNames',secName);
    fname=[outputloc 'IOomg_IndByInd18Sec_1997-2015.xlsx'];
    writetable(omgT,fname,'Sheet',num2str(date),'Range','A1','WriteRowNames',true)
    % par
    par =[alpha,gma,theta,omgE,omgI,omgS,omgR,vaShr];
    parName={'alpha','gamma','theta','Equip','IPP','NonRes S','Res S','VA/GDP'};
    parT=array2table(par,'VariableNames',parName,'RowNames',secName);
    fname=[outputloc 'IOpar_IndByInd18Sec_1997-2015.xlsx'];
    writetable(parT,fname,'Sheet',num2str(date),'Range','A1','WriteRowNames',true)
    
    % Note that we don't have a government sector and that some of the
    % investment requirements are negative (comes from scrap?)    
    % phi: eliminate government, and rescale phi again
    phi=phi(1:N-1,1:N-1);
    phi=phi*diag(1./sum(phi,1));
    phiT=array2table(phi,'VariableNames',secName(1:N-1),'RowNames',secName(1:N-1));
    fname=[outputloc 'IOphi_IndByInd18Sec_1997-2015_final.xlsx'];
    writetable(phiT,fname,'Sheet',num2str(date),'Range','A1','WriteRowNames',true)
    % omg: eliminate gov'mnt, impose non-negativity and rescale omg again
    omg=omg(1:N-1,1:N-1);
    omg=max(omg,zeros(N-1,N-1));
    omg=omg*diag(1./sum(omg,1));
    omgT=array2table(omg,'VariableNames',secName(1:N-1),'RowNames',secName(1:N-1));
    fname=[outputloc 'IOomg_IndByInd18Sec_1997-2015_final.xlsx'];
    writetable(omgT,fname,'Sheet',num2str(date),'Range','A1','WriteRowNames',true)
    % par: eliminate gov'mnt, impose non-negativity and rescale omg of par again
    npar=size(par,2);
    par=max(par(1:N-1,:),zeros(N-1,npar));
    par(:,3:end)=par(:,3:end)*diag(1./sum(par(:,3:end),1));    
    parT=array2table(par,'VariableNames',parName,'RowNames',secName(1:N-1));
    fname=[outputloc 'IOpar_IndByInd18Sec_1997-2015_final.xlsx'];
    writetable(parT,fname,'Sheet',num2str(date),'Range','A1','WriteRowNames',true)
    j=1;
    alpha=par(:,j);j=j+1;
    gma  =par(:,j);j=j+1;
    theta=par(:,j);j=j+1;
    omgE =par(:,j);j=j+1;
    omgI =par(:,j);j=j+1;    
    omgS =par(:,j);j=j+1;
    omgR =par(:,j);j=j+1;
    vaShr=par(:,j);    
    filename = [outputloc 'IO_IndByInd18Sec_' num2str(date) '_final'];
    save(filename,'phi','gma','theta','vaShr','alpha','omgE','omgS','omgI','omgR','omg');
    
end
% -------------------------------------------------------------------------
% Functions used in script
% -------------------------------------------------------------------------
function [xx1,xxRname1,xxCname1]=aggXX(xx,xxRname,xxCname,aggR,aggC,aggIname)
% aggXX aggregated use or make table xx

% no of aggregated industries
nIa=length(aggR);
% no of detailed industries
nI =aggR{nIa}(end);
% size of disaggregated use matrix
[nR,nC]=size(xx);
% size of aggregated use matrix
nRa=nR-nI+nIa;
nCa=nC-nI+nIa;

% Sum of rows: Aggregate commodity use for new commodity definitions
xx1=NaN(nIa,size(xx,2));
for i=1:nIa
    xx1(i,:)=sum(xx(aggR{i},:),1);
end
xx1=[xx1; xx(nI+1:end,:)];
xxRname1=cell(nRa,1);
xxRname1(1:nIa)=aggIname;
xxRname1(nIa+1:nRa)=xxRname(nI+1:nR);

% Sum of columns: Aggregate industry use across new industry definitions 
nIa=length(aggC);
xx=NaN(size(xx1,1),nIa);
for i=1:nIa
    xx(:,i)=sum(xx1(:,aggC{i}),2);
end
xx1=[xx, xx1(:,nI+1:end)];
xxCname1=cell(nCa,1);
xxCname1(1:nIa)=aggIname;
xxCname1(nIa+1:nCa)=xxCname(nI+1:nC);

end


