% model_matrices.m
% print parameter matrices from the model
% 
% Revised: 2021-04-26
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % 

% -- SETUP -- %%
% -- Clear Workspace -- %
clear;
clc;
close all;
tic; 

% -- Set Directories -- %
p = path;

    

% -- Import Data - %
data    = load('../data/KLEMSbea16_1947_1963_1987_2018.mat');
dzv     = data.dzv;
v       = data.v;
n       = data.n;
nq      = data.nq;
N       = 16;
years   = 1950:2016;
nyears  = length(years);

%  value-added shares
vshares     = v./sum(v,2);

% mean labor shares
empshares   = n./sum(n,2); 
mempshares  = mean(n(4:end,:)./sum(n(4:end,:),2))';

% -- Load Model Parameters -- %
load('../data/data_io_cap/IO_2015');             % loading parameters from the Make and Use Tables: theta, phi
load('../data/data_io_cap/capflow');             % loading parameters from the capital flow tables   

% calculating shares from KELMS data
mvashares   = mean(v(4:end,:)./sum(v(4:end,:),2))';

% approximating the zeros in theta
thetanew = theta + (theta==0)*0.0001;
errtheta = sum(thetanew) - 1;
thetanew = thetanew - errtheta./size(thetanew,1);
theta = thetanew;

% Adjust omega for repair and maintenance expenses, McGrattan and Schmitz(1999)
rm = 0.0;
omeganew = omega + rm*eye(size(omega,1));
omeganew = omeganew./(1+rm);
omega = omeganew;
bta = 0.96;             % discount rate
gma_d = diag(gma);
alpha_d = diag(alpha);

capPi = ((eye(N) - phi*(eye(N)-gma_d))/gma_d)';
Xi = ((inv(eye(N) - gma_d*alpha_d*omega' - (eye(N)-gma_d)*phi'))*gma_d)';
    

% -- Table 4: Capital Flow -- %
outfile_name    = 'TableA5.csv';
fileID          = fopen(outfile_name,'w');
for ii = 1:N
    for jj = 1:N
        fprintf(fileID,'%5.2f,',omega(ii,jj));
    end
    fprintf(fileID,'\n');
end
fclose(fileID);

% -- Table 5: Make-Use Flow -- %
outfile_name    = 'TableA6.csv';
fileID          = fopen(outfile_name,'w');
for ii = 1:N
    for jj = 1:N
        fprintf(fileID,'%5.2f,',phi(ii,jj));
    end
    fprintf(fileID,'\n');
end
fclose(fileID);

% -- Table 6: Weighted Leontief Inverse -- %
outfile_name    = 'TableA7.csv';
fileID          = fopen(outfile_name,'w');
for ii = 1:N
    for jj = 1:N
        fprintf(fileID,'%5.2f,',Xi(jj,ii));
    end
    fprintf(fileID,'\n');
end
fclose(fileID);


% -- Table 7: Sectoral Multipliers -- %
outfile_name    = 'Table04.csv';
fileID          = fopen(outfile_name,'w');
indirect        = mvashares'*alpha_d*omega'*Xi';
for ii = 1:N
    fprintf(fileID,'%5.2f, %5.2f, %5.2f \n',[mvashares(ii) indirect(ii) mvashares(ii)+indirect(ii)]);
end
fclose(fileID);

% -- Table A13: Sectoral Multiplier Robustness -- %
mult_2015       = (mvashares'*(eye(N)+alpha_d*omega'*Xi'))';

% different mean shares
mvashares_beg   = mean(v(4:18,:)./sum(v(4:18,:),2))';
mvashares_end   = mean(v(end-14:end,:)./sum(v(end-14:end,:),2))';
mult_beg        = (mvashares_beg'*(eye(N)+alpha_d*omega'*Xi'))';
mult_end        = (mvashares_end'*(eye(N)+alpha_d*omega'*Xi'))';

% 1997 IO Table
load('../data/data_io_cap/IO_1997');             % loading parameters from the Make and Use Tables: theta, phi
gma_d = diag(gma);
alpha_d = diag(alpha);
Xi = ((inv(eye(N) - gma_d*alpha_d*omega' - (eye(N)-gma_d)*phi'))*gma_d)';
mult_1997        = (mvashares'*(eye(N)+alpha_d*omega'*Xi'))';

% 2015 IO Table, 15 sectors
mvashares_15 = mvashares(1:15);
mvashares_15(11) = mvashares_15(11) + mvashares(16);
mvashares_15_beg = mvashares_beg(1:15);
mvashares_15_beg(11) = mvashares_15_beg(11) + mvashares_beg(16);
mvashares_15_end = mvashares_end(1:15);
mvashares_15_end(11) = mvashares_15_end(11) + mvashares_end(16);

load('../data/data_io_cap/capflow_15');
load('../data/data_io_cap/IO_2015_15');             % loading parameters from the Make and Use Tables: theta, phi
gma_d = diag(gma);
alpha_d = diag(alpha);
Xi = ((inv(eye(15) - gma_d*alpha_d*omega' - (eye(15)-gma_d)*phi'))*gma_d)';
mult_2015_15        = (mvashares_15'*(eye(15)+alpha_d*omega'*Xi'))';
mult_2015_15_beg    = (mvashares_15_beg'*(eye(15)+alpha_d*omega'*Xi'))';
mult_2015_15_end    = (mvashares_15_end'*(eye(15)+alpha_d*omega'*Xi'))';

% 1997 IO Table, 15 sectors
load('../data/data_io_cap/capflow_15');
load('../data/data_io_cap/IO_1997_15');             % loading parameters from the Make and Use Tables: theta, phi
gma_d = diag(gma);
alpha_d = diag(alpha);
Xi = ((inv(eye(15) - gma_d*alpha_d*omega' - (eye(15)-gma_d)*phi'))*gma_d)';
mult_1997_15        = (mvashares_15'*(eye(15)+alpha_d*omega'*Xi'))';

% 1960 IO Table
load('../data/data_io_cap/IO_1997_15');             % loading parameters from the Make and Use Tables: theta, phi
load('../data/data_io_cap/IO_1960_15');             % loading parameters from the Make and Use Tables: theta, phi
gma_d = diag(gma);
alpha_d = diag(alpha);
Xi = ((inv(eye(15) - gma_d*alpha_d*omega' - (eye(15)-gma_d)*phi'))*gma_d)';
mult_1960_15 = (mvashares_15'*(eye(15)+alpha_d*omega'*Xi'))';


table = [[mult_2015; mult_2015_15(11)] ...
        [mult_beg; mult_2015_15_beg(11)] ...
        [mult_end; mult_2015_15_end(11)] ...
        [mult_1997; mult_1997_15(11)] ...
        [mult_1960_15(1:10);NaN;mult_1960_15(12:15);NaN; mult_1960_15(11)]];

outfile_name    = 'TableA13.csv';
fileID          = fopen(outfile_name,'w');
for ii = 1:N+1
    fprintf(fileID,'%5.2f, %5.2f, %5.2f, %5.2f, %5.2f \n',table(ii,:));
end
fclose(fileID);
    
% -- END -- %
path(p);
toc;